<template>
  <div></div>
</template>

<script>
import ''
import * as _ from 'lodash'
import Item from './Item.vue'

export default {
  components: {
    Item
  },
  methods: {
    foo() {
      this.
    },
    bar() {
      _.
    }
  },
}
</script>

<style>
</style>
