<template>
  <div class="counter-wrapper">
  </div>
</template>

<script>
export default {
  data () {
    return {
      msg: 'Vetur means "Winter" in icelandic.'
    }
  }
}
</script>

<style lang="scss" scoped>
.counter-wrapper > * {
  margin-bottom: 50px;
}
</style>

