<template>
  <div></div>
</template>

<script>
const lodash = require('lodash')

const foo = 5

export default {

}
</script>

<style lang="scss" scoped>
@import 'src/styles/variables';

div {
  background: red; // popup works here
}

</style>

<style lang="scss">

div {
  background: 'foo';
  font-family: 'Courier New', Courier, monospace;
}

</style>


