<template>
  <div class="counter-wrapper">
    <img src="https://vuejs.org/images/logo.png" width="200" height="200" />
    <a href="./foo">Link to /foo</a>
  </div>
</template>

<script src="./foo.js"></script>
