<template>
  <div class="page">
      <counter>
        

        </counter>
      </div>
</template>

<script>
import Counter from "components/Counter"

export default {
  components: {
  Counter
  }

};

</script>
