<template>
  
</template>

<style>
.foo {
  w
}
</style>

<style lang="less">
.foo {
  w
}
</style>

<style lang="scss">
.foo {
  w
}
</style>

<style lang="stylus">
.foo {
  w
}
</style>

<style lang="postcss">
.foo {
  w
}
</style>

