<template>
  <div>
    <child-comp />
    <
  </div>
</template>

<script>
import ChildComp from '../template/childComponent/ChildComp.vue'

export default {
  components: {
    ChildComp,
    
  }
}
</script>
