<template>
  <div v>
    <i
    <div @scroll.
    <div v-on:scroll.
    <button @keydown.
    <button v-on:keypress.
    <button @click.
    <button v-on:mouseup.
    <item :title.
    <item v-bind:title.
    <input v-model.
    <button @click.="submit()"
  </div>
</template>

<script>
import Item from './Item.vue'

export default {
  components: {
    Item
  }
}
</script>

<style>

</style>
