<template>
  <div></div>
</template>

<script>
/**
 * child component description
 */
export default {
  name: 'childComp',
  props: {
    /**
     * First attr
     */
    'attr-a': {
      type: Number
    }
  }
};
</script>
