<template>
  <div>
    <child />
    <two-s
  </div>
</template>

<script>
import Child from '../template/childComponent/ChildComp.vue'
export default {
  components: {Child},
  data () {
    return {
      foo: ''
    }
  },
  methods: {
    getFoo () {
      return this.
    }
  }
}
</script>
